/**
 * Copyright Copyright 2008-14 Simon Andrews
 *
 *    This file is part of FocalPoint.
 *
 *    FocalPoint is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FocalPoint is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FocalPoint.GUI;

import java.awt.BorderLayout;
import java.awt.Component;
import java.io.File;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.filechooser.FileSystemView;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;

import uk.ac.bbsrc.babraham.FocalPoint.GUI.events.DirectorySelectionListener;

public class DirectoryTree extends JPanel implements TreeSelectionListener {

	// This class presents a windows explorer style view
	// of the directories on a filesystem.  It allows 
	// listeners to attach to be notified when the user
	// selects a new directory.
	
	private JTree dirTree;
	private DefaultTreeModel treeModel;
	private Vector<DirectorySelectionListener> listeners = new Vector<DirectorySelectionListener>();
	private static final FileSystemView fileSystemView = FileSystemView.getFileSystemView();
	
	public DirectoryTree () {
		
		treeModel = new DefaultTreeModel(null);
		dirTree = new JTree(treeModel);
		dirTree.setCellRenderer(new SystemIconRenderer());
		
		File [] roots = fileSystemView.getRoots();
		
		if (roots.length>1) {
			DefaultMutableTreeNode root = new FolderNode(null,"Computer");

			treeModel.setRoot(root);
			
			for (int r=0;r<roots.length;r++) {
				FolderNode n = new FolderNode(roots[r]);
				
				n.setAllowsChildren(true);
				root.add(n);
			}
		}

		else {
			FolderNode n = new FolderNode(roots[0]);
			
			// We need to add the children of this node too.
			
			treeModel.setRoot(n);
		}
		
		dirTree.addTreeSelectionListener(this);

		setLayout(new BorderLayout());
		add(new JScrollPane(dirTree),BorderLayout.CENTER);
	}
	
	public void addListener (DirectorySelectionListener l) {
		if (l != null && ! listeners.contains(l)) {
			listeners.add(l);
		}
	}

	public void removeListener (DirectorySelectionListener l) {
		if (l != null && listeners.contains(l)) {
			listeners.remove(l);
		}
	}

	public void valueChanged(TreeSelectionEvent tse) {

		File f = ((FolderNode)tse.getPath().getLastPathComponent()).getFile();
		if (f != null) {
			Enumeration<DirectorySelectionListener> e = listeners.elements();
			while (e.hasMoreElements()) {
				e.nextElement().directorySelected(f);
			}
		}
		
	}
	
	private class SystemIconRenderer extends DefaultTreeCellRenderer {
		
		public Component getTreeCellRendererComponent (JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {


			Component rc = super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
			if (rc instanceof JLabel && value instanceof FolderNode) {
				JLabel jl = (JLabel) rc;
				File f = ((FolderNode)value).getFile();
				jl.setText(fileSystemView.getSystemDisplayName(f));
				jl.setIcon(fileSystemView.getSystemIcon(f));
			}
			return rc;
		}
	}


	private class FolderNode extends DefaultMutableTreeNode {
		
		private File file;
		private boolean hasBeenChecked = false;

		
		public FolderNode(File file) {
			super(file);
//			System.out.println("Made a node with name "+file.getName()+" with parent "+parent);
			this.file = file;
		}
		
		public FolderNode (File file, String name) {
			super(name);
			this.file = file;
		}
		
		public boolean getAllowsChildren() {
			return true;
		}
		
		public int getChildCount() {
			if (!hasBeenChecked) addChildren();
			return super.getChildCount();
		}
		
		private void addChildren() {
			hasBeenChecked = true;
			if (file.isDirectory()) {
				File[] files = file.listFiles();
				if (files != null) {
					Arrays.sort(files);
					for (int i = 0; i < files.length; i++) {

						/*
						 * Testing a floppy drive using isFile is REALLY slow
						 * hence the extra pre-tests in this case.
						 */
						if (fileSystemView.isFloppyDrive(files[i]) || fileSystemView.isDrive(files[i]) || !files[i].isFile()) {
							/*
							 * It seems daft to check that a file we've just listed actually
							 * exists, but if you have a symlink which doesn't point anywhere
							 * that's exactly what you get.
							 */
							if (!files[i].isHidden() && files[i].exists())
								add(new FolderNode(files[i]));
						}
					}
				}
			}
		}
		
		public File getFile() {
			return file;
		}

		public boolean isLeaf() {
			return false;
		}
		public String toString() {
			if (file != null) {
				if (file.getName().length()>0) {
					return FileSystemView.getFileSystemView().getSystemDisplayName(file);
				}
				else {
					return super.toString();
				}
			}
			else {
				return super.toString();
			}
		}
	}


}
